"""Define the abstract class for Bayesian Neural Networks."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from torch import nn


class BayesianNN(nn.Module):
    """A Bayesian neural network keeps a distribution over neural nets."""

    def __init__(self):
        pass

    def build_model(self):
        pass

    def train(self, data):
        pass

    def sample(self, steps):
        pass



"""Define the abstract class for contextual bandit algorithms."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import numpy as np
from sklearn.metrics import mean_squared_error

def from_pandas(inp):
    try:
        inp = inp.values
    except:
        pass
    return inp

class BanditAlgorithm(object):
    """A bandit algorithm must be able to do two basic operations.

    1. Choose an action given a context.
    2. Update its internal model given a triple (context, played action, reward).
    """

    def action(self, context):
        pass

    def update(self, context, action, reward):
        pass

    def predict_proba(self, context):
        """

        Assumes rewards are associted with a binary outcome
        where r <= 0 is a zero (negative) outcome,
        r > 0 is a one (positive) outcome.

        With this assumption, this function returns the probability
        for a positive outcome for each available action.

        """
        #deal with pandas objects
        context = from_pandas(context)

        expected_values = self.expected_values(context)
        reward_history = self.data_h.rewards
        action_history = np.array(self.data_h.actions)
        for a in range(self.hparams.num_actions):
            action_args = np.argwhere(action_history==a)
            slc = reward_history[action_args,a][:,0]
            convert = np.where(slc > 0, 1, 0)
            pos = convert.sum()
            pos_weight = pos/len(slc)
            center = np.quantile(expected_values[a], (1-pos_weight))
            expected_values[a] = (expected_values[a] - center)/expected_values[a].std()

        def sigmoid(x):
            return 1 / (1 + np.exp(-x))

        probas = sigmoid(expected_values)
        return probas

    def get_sscore(self, context, action, reward):
        """

        Computes the contextual bandits score S
        on some provided validation data
        (context, action, reward triplet).

        Returns a singular score value S.
        0 < S < 1 indicates some convergence (higher is better)
        S < 0 indicates you're better off with a multi-arm bandit model.

        """
        #deal with pandas objects
        context = from_pandas(context)
        action = from_pandas(action)
        reward = from_pandas(reward)

        #recall training data
        past_actions = np.array(self.data_h.actions)
        past_rewards = np.array(self.data_h.rewards)

        #compute naive expected rewards
        E_b = []
        for a in range(self.hparams.num_actions):
            inds = np.argwhere(past_actions==a)
            slc = past_rewards[inds, a][:, 0]
            E = slc.mean()
            E_b.append(E)
        Err_b = []

        #compute benchmark error
        for a in range(self.hparams.num_actions):
            args = np.argwhere(action==a)[:, 0]
            slc = reward[args]
            y_pred = [E_b[a] for x in range(len(slc))]
            y_true = slc
            error = mean_squared_error(y_pred, y_true)
            Err_b.append(error)
        Err_b = np.array(Err_b)

        #compute validation error
        bal = []
        expected_values = self.expected_values(context)
        Err_m = []
        for a in range(self.hparams.num_actions):
            args = np.argwhere(action==a)[:, 0]
            #record representation of action a
            bal.append(len(args))
            y_pred = expected_values[a, args]
            y_true = reward[args]
            E = mean_squared_error(y_pred, y_true)
            Err_m.append(E)
        bal = np.array(bal) / sum(bal)
        rat_vec = 1 - Err_m/Err_b
        avg = np.average(rat_vec, weights=bal)

        return avg



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import numpy as np
from scipy.stats import invgamma
import torch

from .bandit_algorithm import BanditAlgorithm
from .contextual_dataset import ContextualDataset
from .neural_bandit_model import NeuralBanditModel
from .linear import LinearBandits
from .neural_linear import NeuralBandits

import os
import pickle
import shutil

__version__ = '0.0.990'

def load_model(path):
    """loads model from path argument"""
    with open(path, 'rb') as f:
        model = pickle.load(f)
    return model



from distutils.core import setup
import os

long_desc = """
A practical library for building contextual bandits models with deep Bayesian approximation.
Supports both online learning and offline training of models as well as novel methods for cross-validating CB models on historic data.
"""

reqs= [
    'torch',
    'numpy',
    'scipy',
    'pandas',
    'cython',
    'scikit-learn'
]
version = '0.0.992'

setup(
    name='space-bandits',
    version=f'{version}',
    description='Deep Bayesian Contextual Bandits Library',
    long_description=long_desc,
    author='Michael Klear',
    author_email='michael@fellowship.ai',
    url='https://github.com/fellowship/space-bandits',
    download_url=f'https://github.com/fellowship/space-bandits/archive/v{version}.tar.gz',
    install_requires=reqs,
    packages=['space_bandits']
)
